@@ -26,7 +26,7 @@
 #' period-over-period index does nothing.
 #'
 #' Percent-change contributions are removed when chaining/unchaining/rebasing
-#' an index, as it's not usually possible to update them correctly.
+#' an index as it's not usually possible to update them correctly.
 #'
 #' @param x A price index, as made by, e.g., [elemental_index()].
 #' @param link A numeric vector, or something that can coerced into one, of
@@ -34,7 +34,9 @@
 #' that no linking is done.
 #' @param base A numeric vector, or something that can coerced into one, of
 #' base-period index values for each level in `x`. The default is a vector
-#' of 1s so that the base period remains the same.
+#' of 1s so that the base period remains the same. If `base` is a length-one
+#' character vector giving a time period of `x` then the index values for this
+#' time period are used as the base-period values.
 #' @param ... Further arguments passed to or used by methods.
 #'
 #' @returns
@@ -73,36 +75,29 @@ chain.default <- function(x, ...) {
   chain(as_index(x), ...)
 }
 
-#' @export
-chain.aggregate_piar_index <- function(x, ...) {
-  res <- NextMethod("chain")
-  new_aggregate_piar_index(
-    res$index, res$contrib, res$levels,
-    res$time, x$r, x$pias, is_chainable_index(res)
-  )
-}
-
 #' @rdname chain
 #' @export
 chain.chainable_piar_index <- function(x, link = rep(1, nlevels(x)), ...) {
+  chkDots(...)
   link <- as.numeric(link)
   if (length(link) != length(x$levels)) {
     stop("'link' must have a value for each level of 'x'")
   }
   x$index[[1L]] <- x$index[[1L]] * link
   # x$index[] <- Reduce(`*`, x$index, accumulate = TRUE) simplifies results
-  # with one level
-  # TODO: use Reduce once my patch is in a released version of R
+  # with one level.
+  # TODO: use Reduce once my patch is in a released version of R.
   for (t in seq_along(x$time)[-1L]) {
     x$index[[t]] <- x$index[[t]] * x$index[[t - 1L]]
   }
-  # contributions are difficult to chain, so remove them
+  # Contributions are difficult to chain, so remove them.
   x$contrib[] <- empty_contrib(x$levels)
   new_piar_index(x$index, x$contrib, x$levels, x$time, chainable = FALSE)
 }
 
 #' @export
 chain.direct_piar_index <- function(x, ...) {
+  chkDots(...)
   x
 }
 
@@ -118,25 +113,27 @@ unchain.default <- function(x, ...) {
   unchain(as_index(x, chainable = FALSE), ...)
 }
 
-#' @export
-unchain.aggregate_piar_index <- function(x, ...) {
-  res <- NextMethod("unchain")
-  new_aggregate_piar_index(
-    res$index, res$contrib, res$levels,
-    res$time, x$r, x$pias, is_chainable_index(res)
-  )
-}
-
 #' @export
 unchain.chainable_piar_index <- function(x, ...) {
+  chkDots(...)
   x
 }
 
 #' @rdname chain
 #' @export
-unchain.direct_piar_index <- function(x, ...) {
+unchain.direct_piar_index <- function(x, base = rep(1, nlevels(x)), ...) {
+  chkDots(...)
+  if (length(base) == 1L && is.character(base)) {
+    base <- x$index[[match_time(base, x$time)]] / x$index[[1L]]
+  } else {
+    base <- as.numeric(base)
+    if (length(base) != length(x$levels)) {
+      stop("'base' must have a value for each level of 'x'")
+    }
+  }
   x$index[-1L] <- Map(`/`, x$index[-1L], x$index[-length(x$index)])
-  # contributions are difficult to unchain, so remove them
+  x$index[[1L]] <- x$index[[1L]] * base
+  # Contributions are difficult to unchain, so remove them.
   x$contrib[] <- empty_contrib(x$levels)
   new_piar_index(x$index, x$contrib, x$levels, x$time, chainable = TRUE)
 }
@@ -155,18 +152,24 @@ rebase.default <- function(x, ...) {
 
 #' @export
 rebase.chainable_piar_index <- function(x, ...) {
+  chkDots(...)
   x
 }
 
 #' @rdname chain
 #' @export
 rebase.direct_piar_index <- function(x, base = rep(1, nlevels(x)), ...) {
-  base <- as.numeric(base)
-  if (length(base) != length(x$levels)) {
-    stop("'base' must have a value for each level of 'x'")
+  chkDots(...)
+  if (length(base) == 1L && is.character(base)) {
+    base <- x$index[[match_time(base, x$time)]]
+  } else {
+    base <- as.numeric(base)
+    if (length(base) != length(x$levels)) {
+      stop("'base' must have a value for each level of 'x'")
+    }
   }
   x$index <- Map(`/`, x$index, list(base))
-  # contributions are difficult to rebase, so remove them
+  # Contributions are difficult to rebase, so remove them.
   x$contrib[] <- empty_contrib(x$levels)
   x
 }
