@@ -48,6 +48,7 @@
 #' @family aggregation structure methods
 #' @export
 as.matrix.piar_aggregation_structure <- function(x, ..., sparse = FALSE) {
+  chkDots(...)
   nea <- length(x$weights)
   height <- length(x$levels)
   if (height == 1L) {
@@ -59,10 +60,10 @@ as.matrix.piar_aggregation_structure <- function(x, ..., sparse = FALSE) {
     }
   }
   cols <- seq_len(nea)
-  # don't need the eas
+  # Don't need the eas.
   lev <- lapply(as.list(x)[-height], \(z) factor(z, unique(z)))
   res <- vector("list", length(lev))
-  # generate the rows for each level of the matrix and rbind together
+  # Generate the rows for each level of the matrix and rbind together.
   for (i in seq_along(res)) {
     w <- unsplit(
       lapply(split(x$weights, lev[[i]]), gpindex::scale_weights), lev[[i]]
@@ -81,10 +82,13 @@ as.matrix.piar_aggregation_structure <- function(x, ..., sparse = FALSE) {
 
 #' @rdname as.matrix.piar_aggregation_structure
 #' @export
-as.data.frame.piar_aggregation_structure <- function(x, ...,
+as.data.frame.piar_aggregation_structure <- function(x,
+                                                     ...,
                                                      stringsAsFactors = FALSE) {
+  chkDots(...)
   colnames <- c(paste0("level", seq_along(x$child), recycle0 = TRUE), "ea")
-  res <- as.data.frame(as.list(x),
+  res <- as.data.frame(
+    as.list(x),
     col.names = colnames,
     stringsAsFactors = stringsAsFactors
   )
@@ -94,12 +98,13 @@ as.data.frame.piar_aggregation_structure <- function(x, ...,
 
 #' @export
 as.list.piar_aggregation_structure <- function(x, ...) {
+  chkDots(...)
   if (length(x$levels) == 1L) {
     return(x$levels[1L])
   }
   res <- vector("list", length(x$parent))
   res[[1L]] <- x$parent[[1L]]
-  # walk up the parent nodes to reconstruct the inputs that generated 'x'
+  # Walk up the parent nodes to reconstruct the inputs that generated 'x'.
   for (i in seq_along(x$parent)[-1L]) {
     res[[i]] <- x$parent[[i]][res[[i - 1L]]]
   }
