@@ -1,13 +1,17 @@
+#---- Helpers ----
 dim_indices <- function(x, i) {
   if (missing(i)) {
     return(seq_along(x))
   }
   if (is.character(i)) {
-    res <- match(i, x, incomparables = NA)
+    res <- match(i, x)
   } else {
-    res <- match(x[i], x, incomparables = NA)
+    res <- match(x[i], x)
   }
-  if (length(res) == 0L || anyNA(res)) {
+  if (length(res) == 0L) {
+    stop("attempted to select less than one element")
+  }
+  if (anyNA(res)) {
     stop("subscript out of bounds")
   }
   res
@@ -45,7 +49,7 @@ replace_matrix <- function(x, i, value) {
 
   for (i in seq_along(levels)) {
     x$index[[periods[i]]][levels[i]] <- value[(i - 1L) %% n + 1]
-    # drop contributions for replaced values
+    # Drop contributions for replaced values.
     x$contrib[[periods[i]]][levels[i]] <- list(numeric(0L))
   }
   x
@@ -113,12 +117,6 @@ replace_numeric <- function(x, i, j, value) {
 #' is possible to replace value in `x` using a logical matrix or a two-column
 #' matrix of indices.
 #'
-#' Subscripting an aggregate index cannot generally preserve the aggregation
-#' structure if any levels are removed or rearranged, and in this case the
-#' resulting index is *not* an aggregate index. Similarly, replacing the
-#' values for an aggregate index generally breaks consistency in aggregation,
-#' and therefore the result is *not* an aggregate index.
-#'
 #' @param x A price index, as made by, e.g., [elemental_index()].
 #' @param i,j Indices for the levels and time periods of a price index. See
 #' details.
@@ -126,10 +124,7 @@ replace_numeric <- function(x, i, j, value) {
 #' @param ... Not currently used.
 #'
 #' @returns
-#' A price index that inherits from [`chainable_piar_index`] if `x` is a
-#' period-over-period index, or [`direct_piar_index`] if `x` is a
-#' fixed-base index. If `x` inherits from [`aggregate_piar_index`] then
-#' `[` returns an aggregate index if the levels are unchanged.
+#' A price index that inherits from the same class as `x`.
 #'
 #' @examples
 #' index <- as_index(matrix(1:6, 2))
@@ -145,6 +140,7 @@ replace_numeric <- function(x, i, j, value) {
 #' @family index methods
 #' @export
 `[.piar_index` <- function(x, i, j, ...) {
+  chkDots(...)
   periods <- dim_indices(x$time, j)
   # Optimize for extracting by time period.
   if (missing(i)) {
@@ -160,24 +156,12 @@ replace_numeric <- function(x, i, j, value) {
   validate_piar_index(x)
 }
 
-#' @export
-`[.aggregate_piar_index` <- function(x, i, j, ...) {
-  res <- NextMethod("[")
-  if (!identical(res$levels, x$levels)) {
-    new_piar_index(
-      res$index, res$contrib, res$levels, res$time,
-      is_chainable_index(res)
-    )
-  } else {
-    res
-  }
-}
-
 # FIXME: Does it make sense to bundle the contributions with the index values
 # as a list with a dim?
 #' @rdname sub-.piar_index
 #' @export
 `[<-.piar_index` <- function(x, i, j, ..., value) {
+  chkDots(...)
   if (!missing(i) && is.matrix(i)) {
     if (!missing(j)) {
       warning(
@@ -193,15 +177,6 @@ replace_numeric <- function(x, i, j, value) {
   validate_piar_index(x)
 }
 
-#' @export
-`[<-.aggregate_piar_index` <- function(x, i, j, ..., value) {
-  x <- new_piar_index(
-    x$index, x$contrib, x$levels, x$time,
-    is_chainable_index(x)
-  )
-  NextMethod("[<-")
-}
-
 #' @export
 `[<-.chainable_piar_index` <- function(x, i, j, ..., value) {
   if (is_index(value) && !is_chainable_index(value)) {
