@@ -8,9 +8,7 @@
 #' @param ... Not currently used.
 #'
 #' @returns
-#' A price index that inherits from [`chainable_piar_index`] if `x` is a
-#' period-over-period index, or `direct_piar_index()` if `x` is a
-#' fixed-base index.
+#' A price index that inherits from the same class as `x`.
 #'
 #' @examples
 #' index <- as_index(matrix(1:9, 3))
@@ -23,6 +21,7 @@
 #' @importFrom utils head
 #' @export
 head.piar_index <- function(x, n = 6L, ...) {
+  chkDots(...)
   nl <- levels <- length(x$levels)
   np <- periods <- length(x$time)
   if (!is.na(n[1L])) {
@@ -46,6 +45,7 @@ head.piar_index <- function(x, n = 6L, ...) {
 #' @importFrom utils tail
 #' @export
 tail.piar_index <- function(x, n = 6L, ...) {
+  chkDots(...)
   nl <- levels <- length(x$levels)
   np <- periods <- length(x$time)
   if (!is.na(n[1L])) {
