@@ -6,8 +6,8 @@ pias <- as_aggregation_structure(
   data.frame(level1 = 1, level2 = c(11, 12, 13, 14), weight = 1)
 )
 
-epr <- with(dat, elemental_index(rel, period, ea, contrib = TRUE))
-epr2 <- with(dat, elemental_index(rel, period, ea, contrib = FALSE))
+epr <- elemental_index(dat, rel ~ period + ea, contrib = TRUE)
+epr2 <- elemental_index(dat, rel ~ period + ea, contrib = FALSE)
 index <- aggregate(epr, pias)
 
 test_that("as_index makes a valid index", {
@@ -22,6 +22,12 @@ test_that("as_index makes a valid index", {
 
 test_that("as_index works with matrices", {
   expect_equal(as_index(as.matrix(epr)), epr2)
+  expect_equal(
+    contrib(as_index(as.matrix(epr), contrib = TRUE)),
+    as.matrix(epr)[1, , drop = FALSE] - 1
+  )
+  
+  # A character vector used to get pass through without coercion.
   mat <- as.matrix(epr)
   mat[] <- as.character(mat)
   expect_equal(as_index(mat), epr2)
@@ -30,11 +36,18 @@ test_that("as_index works with matrices", {
 
 test_that("as_index works for data frames", {
   expect_equal(as_index(as.data.frame(epr)), epr2)
+  expect_equal(
+    contrib(as_index(as.data.frame(epr), contrib = TRUE)),
+    as.matrix(epr)[1, , drop = FALSE] - 1
+  )
   df <- as.data.frame(epr)
   df[[1]] <- factor(df[[1]], levels = 2:1)
   expect_equal(
     as_index(df),
-    with(dat, elemental_index(rel, factor(period, levels = 2:1), ea))
+    with(
+      dat,
+      elemental_index(rel, period = factor(period, levels = 2:1), ea = ea)
+    )
   )
   
   expect_equal(
