@@ -6,12 +6,11 @@ pias <- as_aggregation_structure(
   data.frame(level1 = 1, level2 = c(11, 12, 13, 14), weight = 1)
 )
 
-epr <- with(
-  dat,
-  elemental_index(setNames(rel, c(1:5, 1, 3, 2, 6)), period, ea, contrib = TRUE)
+epr <- elemental_index(
+  dat, setNames(rel, c(1:5, 1, 3, 2, 6)) ~ period + ea, contrib = TRUE
 )
 index <- aggregate(epr, pias, na.rm = TRUE)
-epr2 <- with(dat, elemental_index(rel, period, ea))
+epr2 <- elemental_index(dat, rel ~ period + ea)
 
 test_that("contrib works", {
   expect_equal(
@@ -34,6 +33,40 @@ test_that("contrib works", {
   )
 })
 
+test_that("contrib2DF works", {
+  expect_equal(
+    contrib2DF(epr),
+    data.frame(
+      period = c("1", "1", "2", "2"),
+      level = c("11", "11", "11", "11"),
+      product = c("1", "2", "1", "2"),
+      value = as.numeric(contrib(epr))
+    )
+  )
+  
+  expect_equal(
+    contrib2DF(epr, levels(epr), 2),
+    data.frame(
+      period = c("2", "2", "2", "2"),
+      level = c("11", "11", "12", "14"),
+      product = c("1", "2", "3", "6"),
+      value = c(
+        contrib(epr, "11", 2), contrib(epr, "12", 2), contrib(epr, "14", 2)
+      )
+    )
+  )
+  
+  expect_equal(
+    contrib2DF(epr2),
+    data.frame(
+      period = character(0),
+      level = character(0),
+      product = character(0),
+      value = numeric(0)
+    )
+  )
+})
+
 test_that("padding work", {
   expect_equal(
     contrib(epr, 12, pad = -99),
@@ -54,7 +87,8 @@ test_that("product names are correct", {
   expect_equal(
     suppressWarnings(
       contrib(elemental_index(c(a = 1, b = 2, c = 3, a = 4, a = 5),
-                              c(1, 1, 1, 2, 2), contrib = TRUE, r = 1))
+        period = c(1, 1, 1, 2, 2), ea = gl(1, 5), contrib = TRUE, r = 1
+      ))
     ),
     matrix(c(0, 0, 1 / 3, 2 / 3, 1.5, 2, 0, 0), 4, 2,
            dimnames = list(c("a", "a.1", "b", "c"), 1:2))
@@ -64,8 +98,10 @@ test_that("product names are correct", {
   expect_equal(
     contrib(epr),
     contrib(
-      with(dat, elemental_index(setNames(rel, c(1:5, 1, 3, 2, 6)),
-                                period, ea, w, contrib = TRUE))
+      elemental_index(dat, rel ~ period + ea,
+        weights = w, contrib = TRUE,
+        product = c(1:5, 1, 3, 2, 6)
+      )
     )
   )
 })
